﻿using System;

namespace David.Academy.ProgrammingInCSharp.Maze
{
	class Program
	{
		enum MazeItem
		{
			None,
			Exit,
			Wall,
			Step,
		}

		static MazeItem[,] CreateMaze(int m, int n)
		{
			MazeItem[,] maze = new MazeItem[m, n];

			Random random = new Random();
			for (int i = 0; i < m; i++)
				for (int j = 0; j < n; j++)
				{
					int number = random.Next(99);
					if (number % 5 == 0)
						maze[i, j] = MazeItem.Wall;
				}

			maze[0, 0] = MazeItem.None;
			maze[m - 1, n - 1] = MazeItem.Exit;

			return maze;
		}

		static void ShowMaze(MazeItem[,] maze, bool isFirstDisplay)
		{
			if (isFirstDisplay)
				Console.Clear();
			else
				Console.SetCursorPosition(0, 0);

			int m = maze.GetLength(0);
			int n = maze.GetLength(1);
			for (int j = 0; j < n; j++)
			{
				for (int i = 0; i < m; i++)
				{
					if (!isFirstDisplay)
						Console.SetCursorPosition(i, j);

					switch (maze[i, j])
					{
						case MazeItem.None:
							Console.Write(' ');
							break;
						case MazeItem.Wall:
							if (isFirstDisplay)
								Console.Write('\u2592');
							break;
						case MazeItem.Step:
							Console.Write('.');
							break;
						case MazeItem.Exit:
							if (isFirstDisplay)
								Console.Write('\u2666');
							break;
					}
				}
				Console.WriteLine();
			}
		}

		static bool TryToStep(MazeItem[,] maze, int x, int y)
		{
			int m = maze.GetLength(0);
			int n = maze.GetLength(1);

			if (x < 0 || x >= m)
				return false;
			if (y < 0 || y >= n)
				return false;

			switch (maze[x, y])
			{
				case MazeItem.None:
					ShowMaze(maze, false);

					maze[x, y] = MazeItem.Step;

					// опит надясно
					if (TryToStep(maze, x + 1, y))
						return true;
					// опит надолу
					if (TryToStep(maze, x, y + 1))
						return true;
					// опит наляво
					if (TryToStep(maze, x - 1, y))
						return true;
					// опит нагоре
					if (TryToStep(maze, x, y - 1))
						return true;

					maze[x, y] = MazeItem.None;

					return false;
				case MazeItem.Wall:
					return false;
				case MazeItem.Step:
					return false;
				case MazeItem.Exit:
					return true;
				default:
					return false;
			}
		}

		static void Main(string[] args)
		{
			Console.CursorVisible = false;
			int m = Console.WindowWidth - 1;
			int n = Console.WindowHeight - 1;
			MazeItem[,] maze = CreateMaze(m, n);
			ShowMaze(maze, true);
			if (TryToStep(maze, 0, 0))
				Console.WriteLine("Yuppie! Prison break!");
			else
				Console.WriteLine("Damn! I'm damned!");
			Console.ReadLine();
		}
	}
}
