﻿using System;
using System.Globalization;

namespace David.Academy.ProgrammingInCSharp.MusicCollection
{
	/// <summary>
	/// Главен клас на програмата
	/// </summary>
	class Program
	{
		/// <summary>
		/// Главна програма
		/// </summary>
		/// <param name="args">Аргументи от командния ред</param>
		static void Main(string[] args)
		{
			// Въвеждане на броя на албумите в колекцията
			int albumCount;
			do
			{
				Console.Write("Брой албуми: ");
				albumCount = int.Parse(Console.ReadLine());
			}
			while (albumCount < 1);


			// Въвеждане на всеки от албумите в колекцията
			Album[] albums = new Album[albumCount];
			for (int i = 0; i < albumCount; i++)
			{
				Console.WriteLine();
				Console.WriteLine("ВЪведете информация за албум №{0}:", i + 1);
				albums[i] = InputAlbum();
			}

			// Изпечатване на информация за всеки от албумите в колекцията
			Console.WriteLine();
			Console.WriteLine("Въведохте следните албуми:");

			for (int i = 0; i < albums.Length; i++)
			{
				Console.WriteLine();
				Console.WriteLine("Албум №{0}:", i + 1);
				OutputAlbum(albums[i]);
			}

			Console.ReadLine();
		}

		/// <summary>
		/// Функция за въвеждане на информация за песен
		/// </summary>
		/// <returns>Обект, представляващ песента</returns>
		static Song InputSong()
		{
			// Деклариране и инстанциране на обект от клас Song
			Song song = new Song();

			// Въвеждане на характеристиките на песента
			Console.Write("Заглавие: ");
			song.Title = Console.ReadLine();
			Console.Write("Изпълнител(и): ");
			song.Artists = Console.ReadLine();
			Console.Write("Автор(и): ");
			song.Authors = Console.ReadLine();
			Console.Write("Продължителност: ");
			song.Duration = TimeSpan.ParseExact(Console.ReadLine(), @"m\:ss", CultureInfo.InvariantCulture);

			return song;
		}

		/// <summary>
		/// Функция за въвеждане на характеристиките на албум и песните в него
		/// </summary>
		/// <returns>Обект, представляващ албума</returns>
		static Album InputAlbum()
		{
			// Деклариране и инстанциране на обект от клас Album
			Album album = new Album();

			// Въвеждане на характеристиките на албума
			Console.Write("Заглавие: ");
			album.Title = Console.ReadLine();
			Console.Write("Изпълнител(и): ");
			album.Artists = Console.ReadLine();
			Console.Write("Година на издаване: ");
			album.Year = int.Parse(Console.ReadLine());
			Console.Write("Формат (Vinyl, Tape, CD, Flac): ");
			album.Format = (AlbumFormat)Enum.Parse(typeof(AlbumFormat), Console.ReadLine());

			// Въвеждане на броя на песните в албума
			int songCount;
			do
			{
				Console.Write("Брой песни: ");
				songCount = int.Parse(Console.ReadLine());
			}
			while (songCount < 1);

			// Въвеждане на всяка от песните в албума
			album.Songs = new Song[songCount];
			for (int i = 0; i < songCount; i++)
			{
				Console.WriteLine();
				Console.WriteLine("ВЪведете информация за песен №{0}:", i + 1);
				album.Songs[i] = InputSong();
			}

			return album;
		}

		/// <summary>
		/// Функция за изпечатване на характеристиките на песен
		/// </summary>
		/// <param name="song">Обект, представляващ песента</param>
		static void OutputSong(Song song)
		{
			// Изпечатване на информация за песента
			Console.WriteLine("Заглавие: {0}", song.Title);
			Console.WriteLine("Изпълнител(и): {0}", song.Artists);
			Console.WriteLine("Автор(и): {0}", song.Authors);
			Console.WriteLine(@"Продължителност: {0:m\:ss}", song.Duration);
		}

		/// <summary>
		/// Функция за изпечатване на характеристиките на албум и песните в него
		/// </summary>
		/// <param name="album">Обект, представляващ албума</param>
		static void OutputAlbum(Album album)
		{
			// Изпечатване на информация за албума
			Console.WriteLine("Заглавие: {0}", album.Title);
			Console.WriteLine("Изпълнител(и): {0}", album.Artists);
			Console.WriteLine("Година на издаване: {0}", album.Year);
			Console.WriteLine("Формат: {0}", album.Format);

			// Изпечатване на информация за всяка от песните в албума
			for (int i = 0; i < album.Songs.Length; i++)
			{
				Console.WriteLine();
				Console.WriteLine("Песен №{0}:", i + 1);
				OutputSong(album.Songs[i]);
			}
		}
	}
}
