﻿using System;

namespace David.Academy.ProgrammingInCSharp.Encapsulation
{
	/// <summary>
	/// Клас за правоъгълник. Наследява класа Shape2D
	/// </summary>
	class Rectangle : Shape2D
	{
		// Частни полета за дължини на страните на правоъгълника
		private double _sideA;
		private double _sideB;

		/// <summary>
		/// Конструктор на правоъгълника с параметри: дължини на страните му;
		/// изпълнява защитения конструктор на родителския клас със съответната
		/// стойност за лице на правоъгълника
		/// </summary>
		/// <param name="sideA">Дължина на първата страна на правоъгълника</param>
		/// <param name="sideB">Дължина на втората страна на правоъгълника</param>
		public Rectangle(double sideA, double sideB)
			: base(sideA * sideB)
		{
			// Предизвикване на грешка при изпълнение, ако е подадена неположителна дължина на страна
			if (sideA <= 0)
				throw new ArgumentException();
			if (sideB <= 0)
				throw new ArgumentException();

			_sideA = sideA;
			_sideB = sideB;
		}

		/// <summary>
		/// Връща дължината на първата страна на правоъгълника
		/// </summary>
		/// <returns>Дължина на първата страна на правоъгълника</returns>
		public double GetSideA()
		{
			return _sideA;
		}

		/// <summary>
		/// Връща дължината на втората страна на правоъгълника
		/// </summary>
		/// <returns>Дължина на втората страна на правоъгълника</returns>
		public double GetSideB()
		{
			return _sideB;
		}

		/// <summary>
		/// Променя дължината на първата страна на правоъгълника
		/// </summary>
		/// <param name="sideA">Нова дължина на първата страна на правоъгълника</param>
		public void SetSideA(double sideA)
		{
			// Предизвикване на грешка при изпълнение, ако е подадена неположителна дължина на страна
			if (sideA <= 0)
				throw new ArgumentException();

			_sideA = sideA;
			// Преизчисляване на лицето на правоъгълника
			SetArea(_sideA * _sideB);
		}

		/// <summary>
		/// Променя дължината на втората страна на правоъгълника
		/// </summary>
		/// <param name="sideB">Нова дължина на втората страна на правоъгълника</param>
		public void SetSideB(double sideB)
		{
			// Предизвикване на грешка при изпълнение, ако е подадена неположителна дължина на страна
			if (sideB <= 0)
				throw new ArgumentException();

			_sideB = sideB;
			// Преизчисляване на лицето на правоъгълника
			SetArea(_sideA * _sideB);
		}
	}
}
