﻿using System;

namespace David.Academy.ProgrammingInCSharp.Inheritance
{
	/// <summary>
	/// Клас за обществена сграда. Наследява класа Building
	/// </summary>
	class CivicBuilding : Building
	{
		// Характеристики, специфични за обществените сгради:
		// работно време и дали сградата е частна собственост
		public TimeSpan OpeningHoursStart;
		public TimeSpan OpeningHoursEnd;
		public bool IsPrivate;

		/// <summary>
		/// Конструктор с параметри: височина, заемана площ и дали сградата е частна собственост;
		/// изпълнява конструктора на родителския клас с параметри височина и заемана площ
		/// </summary>
		/// <param name="height">Височина на сградата</param>
		/// <param name="area">Заемана площ</param>
		/// <param name="isPrivate">Указва дали сградата е частна собственост</param>
		public CivicBuilding(double height, double area, bool isPrivate)
			: base(height, area)
		{
			IsPrivate = isPrivate;
		}

		/// <summary>
		/// Променя работното време на обществената сграда; проверява дали подадените стойности
		/// на параметрите са валидни
		/// </summary>
		/// <param name="start">Начален час на работното време (00:00 - 24:00)</param>
		/// <param name="end">Краен час на работното време (00:00 - 24:00)</param>
		public void ChangeOpeningHours(TimeSpan start, TimeSpan end)
		{
			if (start < end
				&& start >= TimeSpan.Zero
				&& start <= TimeSpan.FromHours(24)
				&& end >= TimeSpan.Zero
				&& end <= TimeSpan.FromHours(24))
			{
				OpeningHoursStart = start;
				OpeningHoursEnd = end;
			}
		}
	}
}
