﻿using System;

namespace David.Academy.ProgrammingInCSharp.Inheritance
{
	/// <summary>
	/// Главен клас на програмата
	/// </summary>
	class Program
	{
		/// <summary>
		/// Главна програма
		/// </summary>
		/// <param name="args">Аргументи от командния ред</param>
		static void Main(string[] args)
		{
			#region Примери от група 1

			// Създаване на екземпляр на наследения клас ResidentialBuilding
			// и присвояването му на променлива от същия тип
			ResidentialBuilding residentialBuilding = new ResidentialBuilding(10.5, 2500, 2, 40);

			// Достъпване на полета, декларирани от родителския клас
			// (на практика се осъществява upcasting към тип Building)
			residentialBuilding.Height = 10.5;
			residentialBuilding.Area = 2500;
			residentialBuilding.FloorCount = 3;

			// Достъпване на полета, декларирани от наследения клас
			residentialBuilding.EntranceCount = 2;
			residentialBuilding.ResidentCount = 40;

			// Извикване на метод, деклариран от наследения клас
			residentialBuilding.MovePeopleIn(5);

			// Извикване на метод, деклариран от родителския клас
			// (на практика се осъществява upcasting към тип Building)
			residentialBuilding.Demolish();

			// Извикване на методи, декларирани от класа object
			// (на практика се осъществява upcasting към тип object)
			residentialBuilding.GetType();
			residentialBuilding.ToString();

			// Извикване на метода GetDescription(), деклариран от наследения клас
			// (методът, деклариран от наследения клас, "скрива" декларирания от родителския клас)
			Console.WriteLine(residentialBuilding.GetDescription());

			// Присвояване на екземпляр на наследения клас на променлива, чийто тип
			// е родителският клас (upcasting)
			Building building = residentialBuilding;
			building.Height = 21;

			// Downcasting на реферираната стойност към наследения клас в сложен израз
			// Няма да предизвика грешка, тъй като обектът в паметта е от съвместим тип
			((ResidentialBuilding)building).EntranceCount = 4;

			Console.WriteLine(building.GetDescription());
			Console.WriteLine(((Building)residentialBuilding).GetDescription());

			// Присвояване на реферираната стойност на променлива, чийто тип е наследеният клас (downcasting)
			// Няма да предизвика грешка, тъй като обектът в паметта е от съвместим тип
			ResidentialBuilding myBuilding = (ResidentialBuilding)building;
			Console.WriteLine("Height: {0}", myBuilding.Height);
			Console.WriteLine("Residents: {0}", myBuilding.ResidentCount);
			Console.WriteLine("Entrances: {0}", ((ResidentialBuilding)building).EntranceCount);

			// Присвояване на реферираната стойност на променлива, чийто тип е друг наследен клас (downcasting):
			// Ще предизвика грешка при изпълнение, тъй като обектът в паметта НЕ е от съвместим тип
			//CivicBuilding civicBuilding = (CivicBuilding)building;

			// Опит за присвояване на обект, рефериран от израз от тип ResidentialBuilding, на променлива
			// от тип CivicBuilding
			// Ще предизвика грешка при компилация, тъй като компилаторъ може да разпознае,
			// че типовете CivicBuilding и ResidentialBuilding са от различни клонове на йерархията
			// (нито един от тях не е в йерархията от родителски типове на другия)
			//CivicBuilding civicBuilding = (CivicBuilding)residentialBuilding;

			// Присвояване на нов екземпляр на класа CivicBuilding на съществуващата променлива,
			// чийто тип е родителският клас
			building = new CivicBuilding(50, 10000, true);

			// Проверка за съвместимост на типове за безопасен downcasting с оператора as
			if (building is CivicBuilding)
			{
				// Downcasting към наследения тип CivicBuilding след проверката за безопасност
				Console.WriteLine("IsPrivate: {0}", ((CivicBuilding)building).IsPrivate);
			}

			// Безопасен dowcasting към наследения тип CivicBuilding с оператора as
			CivicBuilding civicBuilding = building as CivicBuilding;
			if (civicBuilding != null)
				Console.WriteLine("IsPrivate: {0}", civicBuilding.IsPrivate);

			#endregion

			#region Примери от група 2

			// Създаване на екземпляр на наследения клас Planet и присвояването му на
			// променлива от същия тип
			Planet earth = new Planet("Earth", 1);

			// Достъпване на полета, декларирани от родителския клас
			// (на практика се осъществява upcasting към тип CelestialBody)
			//earth.Name = "Earth";
			earth.Mass = 5.97e+21;
			earth.Distance = 0;

			// Достъпване на полета, декларирани от наследения клас
			earth.IsInhabited = true;
			earth.Type = "Rocky";
			earth.HasRings = false;
			//earth.SatelliteCount = 1;

			// Извикване на метода GetDescription(), деклариран от наследения клас
			// (методът, деклариран от наследения клас, "скрива" декларирания от родителския клас)
			Console.WriteLine(earth.GetDescription());

			// Присвояване на екземпляр на наследения клас на променлива, чийто тип
			// е родителският клас (upcasting)
			CelestialBody body = earth;

			// Извикване на метода GetDescription(), деклариран от родителския клас
			Console.WriteLine(body.GetDescription());

			// Извикване на метода GetDescription(), деклариран от наследения клас,
			// посредством downcasting в сложен израз; няма да предизвика грешка, тъй като
			// обектът в паметта е от съвместим тип
			Console.WriteLine(((CelestialBody)earth).GetDescription());

			// Създаване на екземпляр на наследения клас Star и присвояването му на
			// променлива от съшщия тип
			Star sun = new Star();

			// Опит за присвояване на обект, рефериран от израз от тип Sun, на променлива от тип Planet
			// Ще предизвика грешка при компилация, тъй като компилаторът може да разпознае, че
			// типовете Planet и Star са от различни клонове на йерархията (нито един от тях
			// не е в йерархията от родителски типове на другия)
			//Planet planet2 = (Planet)sun;

			// Изпечатва текстовото описание на обекта от тип Planet
			Print(earth);
			// Изпечатва текстовото описание на обекта от тип Star
			Print(sun);
			// Изпечатва текстовото описание на новосъздаден екземпляр на родителския клас CelestialBody
			Print(new CelestialBody());

			// Въвеждане на планета или звезда от клавиатурата
			CelestialBody newBody = null;
			Console.Write("Would you like to create a planet (P) or a star (S): ");
			switch (Console.ReadKey().KeyChar)
			{
				case 'p':
					newBody = InputPlanet();
					break;
				case 's':
					newBody = InputStar();
					break;
			}

			// Изпечатване на винформация за въведенето небесно тяло
			if (newBody != null)
				Print(newBody);

			#endregion

			Console.ReadLine();
		}

		/// <summary>
		/// Изпечатва на екрана текстовото описание на произволно небесно тяло,
		/// но извежда и специфичните за наследените класове характеристики
		/// </summary>
		/// <param name="body">Небесно тяло</param>
		public static void Print(CelestialBody body)
		{
			// Проверки за съвместимост на типове и безопасен downcasting към съответния тип,
			// за да бъде изпечатано пълното описание на характеристиките на небесното тяло
			if (body is Planet)
				Console.WriteLine(((Planet)body).GetDescription());
			else if (body is Star)
				Console.WriteLine(((Star)body).GetDescription());
			else
				Console.WriteLine(body.GetDescription());
		}

		/// <summary>
		/// Въвежда от клавиатурата информация за планета
		/// </summary>
		/// <returns>Новосъздадената планета</returns>
		public static Planet InputPlanet()
		{
			Console.Write("Name: ");
			string name = Console.ReadLine();
			Console.Write("Satellites: ");
			int satelliteCount = int.Parse(Console.ReadLine());
			Planet planet = new Planet(name, satelliteCount);
			Console.Write("Mass: ");
			planet.Mass = double.Parse(Console.ReadLine());
			Console.Write("Distance: ");
			planet.Distance = double.Parse(Console.ReadLine());
			Console.Write("Is inhabited: ");
			planet.IsInhabited = bool.Parse(Console.ReadLine());
			Console.Write("Type: ");
			planet.Type = Console.ReadLine();
			Console.Write("Has rings: ");
			planet.HasRings = bool.Parse(Console.ReadLine());
			return planet;
		}

		/// <summary>
		/// Въвежда от клавиатурата информация за звезда
		/// </summary>
		/// <returns>Новосъздадената звезда</returns>
		public static Star InputStar()
		{
			Star star = new Star();
			Console.Write("Name: ");
			star.Name = Console.ReadLine();
			Console.Write("Mass: ");
			star.Mass = double.Parse(Console.ReadLine());
			Console.Write("Distance: ");
			star.Distance = double.Parse(Console.ReadLine());
			Console.Write("Luminosity: ");
			star.Luminosity = double.Parse(Console.ReadLine());
			Console.Write("Color: ");
			star.Color = Console.ReadLine();
			Console.Write("Radius: ");
			star.Radius = double.Parse(Console.ReadLine());
			return star;
		}
	}
}
