﻿namespace David.Academy.ProgrammingInCSharp.Indexers
{
	/// <summary>
	/// Клас за библиотека
	/// </summary>
	class Library
	{
		// Частен масив от книги
		private readonly Book[] _books;

		/// <summary>
		/// Брой книги в библиотеката
		/// </summary>
		public int BookCount { get; private set; }

		/// <summary>
		/// Капацитет на библиотеката (максимален брой книги)
		/// </summary>
		public int Capacity
		{
			get { return _books.Length; }
		}

		/// <summary>
		/// Индексатор за достъпване на книга по номера и в колекцията;
		/// Номерацията започва от 1
		/// </summary>
		/// <param name="index">Номер на книгата в колекцията</param>
		/// <returns>Книгата със съответния номер или null, в случай че номерът е невалиден</returns>
		public Book this[int index]
		{
			// Блок за прочитане на елемент с индексатора
			get
			{
				if (index >= 1 || index <= BookCount)
					return _books[index - 1];
				else
					return null;
			}
			// Блок за промяна на елемент с индексатора
			//set { _books[index - 1] = value; }
		}

		/// <summary>
		/// Индексатор само за четене за достъпване на книга по заглавието ѝ
		/// </summary>
		/// <param name="title">Заглавие на книгата</param>
		/// <returns>Книгата със съответното заглавие или null, в случай че такава не съществува</returns>
		public Book this[string title]
		{
			// Блок за прочитане на елемент с индексатора
			get
			{
				foreach (Book book in _books)
				{
					if (book == null)
						continue;
					if (book.Title == title)
						return book;
				}
				return null;
			}
		}

		/// <summary>
		/// Конструктор на библиотеката с параметър капацитет
		/// </summary>
		/// <param name="capacity">Капацитет на библиотеката</param>
		public Library(int capacity)
		{
			_books = new Book[capacity];
		}

		/// <summary>
		/// Метод за добавяне на книга в библиотеката
		/// </summary>
		/// <param name="book">Книга, която трябва да бъде добавена в библиотеката</param>
		public void AddBook(Book book)
		{
			if (BookCount < Capacity)
			{
				_books[BookCount] = book;
				BookCount++;
			}
		}
	}
}
