﻿using System;

namespace David.Academy.ProgrammingInCSharp.StaticMembers
{
	/// <summary>
	/// Клас за физическо лице
	/// </summary>
	class Person
	{
		// Частно поле самое за четене за съхраняване на поредния номер на лицето
		private readonly int _personNumber;

		// Статично поле за последния генериран пореден номер на лице;
		// Заменено с ендоименното статично свойство по-долу
		//public static int LastPersonNumber;

		/// <summary>
		/// Статично свойство за последния генериран пореден номер на лице
		/// </summary>
		public static int LastPersonNumber { get; private set; }

		/// <summary>
		/// Име на лицето
		/// </summary>
		public string Name { get; private set; }

		/// <summary>
		/// Адрес на лицето
		/// </summary>
		public string Address { get; private set; }

		/// <summary>
		/// Пореден номер на лицето - генерира се автоматично при създаване на обекта
		/// </summary>
		public int PersonNumber
		{
			get { return _personNumber; }
		}

		/// <summary>
		/// Статичен конструктор на класа; Изпълнява се непосредствено преди първата
		/// употреба на класа в програмата
		/// </summary>
		static Person()
		{
			Console.Write("Input initial person number: ");
			LastPersonNumber = int.Parse(Console.ReadLine()) - 1;
		}

		/// <summary>
		/// Контруктор на лице с параметри име и адрес;
		/// Автоматично присвоява на лицето уникален пореден номер с помощта на статичното
		/// свойство LastPersonNumber
		/// </summary>
		/// <param name="name">Име на лицето</param>
		/// <param name="address">Адрес на лицето</param>
		public Person(string name, string address)
		{
			Name = name;
			Address = address;

			LastPersonNumber++;
			_personNumber = LastPersonNumber;
		}

		/// <summary>
		/// Статичен метод за проверка дали указаният пореден номер на лице е все още
		/// неразпределен от програмата
		/// </summary>
		/// <param name="personNumber">Пореден номер на лице</param>
		/// <returns>Истина, ако указаният пореден номер на лице все още не е разпределен
		/// на някой от вече конструираните обекти и може да бъде разпределен в бъдеще</returns>
		public static bool IsPersonNumberAvailable(int personNumber)
		{
			return (personNumber > LastPersonNumber && personNumber < 1000000);
		}

		/// <summary>
		/// Статичен метод за въвеждане на ново лице от конзолата
		/// </summary>
		/// <returns></returns>
		public static Person InputFromKeyboard()
		{
			Console.Write("Input person name: ");
			string name = Console.ReadLine();
			Console.Write("Input person address: ");
			string address = Console.ReadLine();
			return new Person(name, address);
		}
	}
}
