﻿using System.Collections;

namespace David.Academy.ProgrammingInCSharp.Enumerations
{
	/// <summary>
	/// Клас за колекция от песни. Имплементира интерфейса IEnumerable
	/// </summary>
	class SongCollection : IEnumerable
	{
		// Частен масив от песни
		private readonly Song[] _songs;
		// Брой песни в колекцията
		private int _count;

		/// <summary>
		/// Вътрешен клас за изброяване на колекцията от песни. Имплементира интерфейса IEnumerator
		/// </summary>
		private class Enumerator : IEnumerator
		{
			// Частна референция към колекцията от песни, която се изброява
			private readonly SongCollection _collection;
			// Индекс на елемента, върху който се намира изброяващият обект в момента
			private int _index;

			/// <summary>
			/// Конструктор на обекта за изброяване на колекцията от песни
			/// </summary>
			/// <param name="collection">Колекция от песни, която ще се изброява</param>
			public Enumerator(SongCollection collection)
			{
				_collection = collection;
				_index = -1;
			}

			/// <summary>
			/// Връща елемента от колекцията от песни, върху който се намира изброяващият обект в момента
			/// </summary>
			public object Current
			{
				get { return _collection._songs[_index]; }
			}

			/// <summary>
			/// Придвижва обекта за изброяване на колекцията от песни върху следващия елемент
			/// или след края на колекцията, в случай че няма такъв
			/// </summary>
			/// <returns>Истина, ако обектът за изброяване на колекцията е бил преместен успешно
			/// върху следващия елемент; Неистина, ако няма повече елементи в колекцията</returns>
			public bool MoveNext()
			{
				_index++;
				return (_index < _collection._count);
			}

			/// <summary>
			/// Връща обекта за изброяване на колекцията от песни в начално състояние
			/// (т.е. непосредствено преди първия елемент в колекцията)
			/// </summary>
			public void Reset()
			{
				_index = -1;
			}
		}

		/// <summary>
		/// Конструктор на колекцията от песни с параметър капацитет на колекцията
		/// </summary>
		/// <param name="capacity">Капацитет на колекцията</param>
		public SongCollection(int capacity)
		{
			_songs = new Song[capacity];
		}

		/// <summary>
		/// Добавя песен в колекцията
		/// </summary>
		/// <param name="book">Песен, която трябва да бъде добавена в колекцията</param>
		public void AddSong(Song song)
		{
			_songs[_count++] = song;
		}

		/// <summary>
		/// Връща обект за изброяване на колекцията от песни
		/// </summary>
		/// <returns>Обект за изброяване на колекцията от песни, който имплементира интерфейса
		/// IEnumerator</returns>
		public IEnumerator GetEnumerator()
		{
			return new Enumerator(this);
		}
	}
}
