﻿using System;

namespace David.Academy.ProgrammingInCSharp.Generics
{
	/// <summary>
	/// Шаблонен клас за възел от строготипизирано наредено двоично дърво
	/// </summary>
	/// <typeparam name="T">Тип на данните в дървото; трябва да имплементира интерфейса
	/// IComparable, така че данните на възлите да могат да бъдат сравнявани
	/// и съответно наредени</typeparam>
	class BinaryTreeNode<T>
		where T : IComparable
	{
		// Частно поле за съхраняване на данните на възела
		private readonly T _data;

		/// <summary>
		/// Данни на възела
		/// </summary>
		public T Data
		{
			get { return _data; }
		}

		/// <summary>
		/// Ляв дъщерен възел
		/// </summary>
		public BinaryTreeNode<T> Left { get; private set; }

		/// <summary>
		/// Десен дъщерен възел
		/// </summary>
		public BinaryTreeNode<T> Right { get; private set; }

		/// <summary>
		/// Конструктор на възела с параметър данни, които да бъдат съхранени в него
		/// </summary>
		/// <param name="data">Данни, които да бъдат съхранени във възела</param>
		public BinaryTreeNode(T data)
		{
			_data = data;
		}

		/// <summary>
		/// Добавя нов възел в нареденото двоично дърво, чийто корен е текущият възел
		/// </summary>
		/// <param name="data">Данни на новия възел</param>
		public void AddNode(T data)
		{
			// Използва се методът IComparable.CompareTo(object), за да се сравнят данните
			// в текущия възел и данните на новия възел;
			// Това е възможно, тъй като е наложено изискването типът на данните T да имплементира
			// интерфейса IComparable
			if (data.CompareTo(_data) <= 0)
			{
				if (Left == null)
					Left = new BinaryTreeNode<T>(data);
				else
				{
					// Ако текущият възел вече има ляв дъщерен възел, се извиква рекурсивно методът
					// AddNode(T) върху него
					Left.AddNode(data);
				}
			}
			else
			{
				if (Right == null)
					Right = new BinaryTreeNode<T>(data);
				else
				{
					// Ако текущият възел вече има десен дъщерен възел, се извиква рекурсивно методът
					// AddNode(T) върху него
					Right.AddNode(data);
				}
			}
		}
	}
}
