﻿using System;

namespace David.Academy.ProgrammingInCSharp.Interfaces
{
	/// <summary>
	/// Главен клас на програмата
	/// </summary>
	class Program
	{
		/// <summary>
		/// Главна програма
		/// </summary>
		/// <param name="args">Аргументи от командния ред</param>
		static void Main(string[] args)
		{
			// Създаване на екземпляр на класа Tractor
			Tractor tractor = new Tractor();
			// Извикване на метода MakeSound() на променливата от тип Tractor
			tractor.MakeSound();
			// Създаване на екземпляр на класа Trabant
			Trabant trabant = new Trabant();
			// Извикване на метода MakeSound() на променливата от тип Trabant
			trabant.MakeSound();

			// Създаване на еземпляр на структурата Point
			Point p = new Point
			{
				X = 10,
				Y = -0.33,
			};
			// Създаване на еземпляр на класа StealthPlane
			StealthPlane plane = new StealthPlane();
			// Опит за извикване на метода MoveTo(Point) на променливата от тип StealthPlane;
			// Предизвиква грешка при компилация, тъй като методът MoveTo(Point) на интерфейса IMobileObject
			// е имплементиран явно в класа StealthPlane
			//plane.MoveTo(p);

			// Създаване на еземпляр на класа Song
			Song song = new Song("Lightning Bolt", "Pearl Jam");
			// Извикване на метода PrintToConsole() на променливата от тип Song
			song.PrintToConsole();
			// Създаване на еземпляр на класа Alarm
			Alarm alarm = new Alarm(DateTime.Now.AddHours(1));
			// Извикване на метода PrintToConsole() на променливата от тип Alarm
			alarm.PrintToConsole();
			// Опит за извикване на метода MakeSound() на променливата от тип Alarm;
			// Предизвиква грешка при компилация, тъй като методът MakeSound() на интерфейса ISoundMaker
			// е имплементиран явно в класа Alarm
			//alarm.MakeSound();

			// Изпълнение на статичния метод Program.MakeSound(ISoundMaker) с екземпляри на няколко различни класа
			MakeSound(tractor);
			MakeSound(trabant);
			MakeSound(alarm);
			// Изпълнение на статичния метод Program.TryMoveTo(IMobileObject, Point) с екземпляри на няколко различни класа
			TryMoveTo(plane, p);
			TryMoveTo(tractor, p);
			// Изпълнение на статичния метод Program.Print(IPrintable) с екземпляри на няколко различни класа
			Print(song);
			Print(alarm);
			// Опит за изпълнение на статичния метод Program.Print(IPrintable) с променлива от тип Tractor,
			// чийто тип е явно преобразуван към IPrintable;
			// Предизвиква грешка по време на изпълнение, чъй като обектът в паметта, който променливата реферира,
			// не имплементира интерфейса IPrintable
			//Print((IPrintable)tractor);

			// Присвояване на екземпляр на класа Tractor на променлива от тип ISoundMaker
			ISoundMaker soundMaker = tractor;
			// Изпълнение на статичния метод Program.TryMoveTo(IMobileObject, Point) с променлива от тип ISoundMaker,
			// чийто тип е явно преобразуван към IMobileObject;
			// Не предизвиква грешка по време на компилация или изпълнение, тъй като обектът в паметта, който променливата
			// реферира, е екземпляр на класа Tractor и имплементира интерфейса IMobileObject
			TryMoveTo((IMobileObject)soundMaker, p);

			Console.ReadLine();
		}

		/// <summary>
		/// Кара обект, имплементиращ интерфейса ISoundMaker, да издаде специфичния за него звук
		/// </summary>
		/// <param name="soundMaker">Обект, имплементиращ интерфейса ISoundMaker</param>
		private static void MakeSound(ISoundMaker soundMaker)
		{
			soundMaker.MakeSound();
		}

		/// <summary>
		/// Придвижва обект, имплементиращ интерфейса IMobileObject, до указаната точка, ако това е възможно
		/// </summary>
		/// <param name="mobileObject">Обект, имплементиращ интерфейса IMobileObject</param>
		/// <param name="location">Точка, до която трябва да бъде придвижен обекта</param>
		/// <returns>Истина, в случай че обектът е бил придвижен успешно</returns>
		private static bool TryMoveTo(IMobileObject mobileObject, Point location)
		{
			if (!mobileObject.CanMoveTo(location))
				return false;

			mobileObject.MoveTo(location);
			return true;
		}

		/// <summary>
		/// Изпечатва на конзолата характеристиките на обект, имплементиращ интефейса IPrintable
		/// </summary>
		/// <param name="printableObject">Обект, имплементиращ интерфейса IPrintable</param>
		static void Print(IPrintable printableObject)
		{
			printableObject.PrintToConsole();
		}
	}
}
