﻿using System.Collections.Generic;

namespace David.Academy.ProgrammingInCSharp.Exceptions
{
	class BookDatabaseV1
	{
		private readonly List<BookEntry> _entries;

		public BookDatabaseV1()
		{
			_entries = new List<BookEntry>();
		}

		public List<BookEntry> GetEntries(out BookDatabaseStatus status)
		{
			List<BookEntry> entries = new List<BookEntry>();
			foreach (BookEntry entry in _entries)
			{
				entries.Add(entry);
			}

			status = BookDatabaseStatus.Ok;

			return entries;
		}

		public BookEntry GetEntry(int index, out BookDatabaseStatus status)
		{
			if (index < 0 || index > _entries.Count - 1)
			{
				status = BookDatabaseStatus.ArgumentError;
				return null;
			}

			status = BookDatabaseStatus.Ok;

			return _entries[index];
		}

		public void AddEntry(BookEntry entry, out BookDatabaseStatus status)
		{
			status = BookDatabaseStatus.Ok;

			_entries.Add(entry);
		}

		public void RemoveEntry(int index, out BookDatabaseStatus status)
		{
			if (index < 0 || index > _entries.Count - 1)
			{
				status = BookDatabaseStatus.ArgumentError;
				return;
			}

			status = BookDatabaseStatus.Ok;

			_entries.RemoveAt(index);
		}

		public List<BookEntry> FindEntries(string name, out BookDatabaseStatus status)
		{
			List<BookEntry> entries = new List<BookEntry>();
			foreach (BookEntry entry in _entries)
			{
				if (entry.Name.Contains(name))
					entries.Add(entry);
			}

			status = BookDatabaseStatus.Ok;

			return entries;
		}
	}
}
