﻿using System.Collections.Generic;

namespace David.Academy.ProgrammingInCSharp.Exceptions
{
	class BookDatabaseV3
	{
		private readonly List<BookEntry> _entries;

		public BookDatabaseV3()
		{
			_entries = new List<BookEntry>();
		}

		public List<BookEntry> GetEntries()
		{
			List<BookEntry> entries = new List<BookEntry>();
			foreach (BookEntry entry in _entries)
			{
				entries.Add(entry);
			}

			return entries;
		}

		public BookEntry GetEntry(int index)
		{
			if (index < 0 || index > _entries.Count - 1)
				throw new BookDatabaseException(BookDatabaseStatus.ArgumentError, "Index is out of range.");

			return _entries[index];
		}

		public void AddEntry(BookEntry entry)
		{
			_entries.Add(entry);
		}

		public void RemoveEntry(int index)
		{
			if (index < 0 || index > _entries.Count - 1)
				throw new BookDatabaseException(BookDatabaseStatus.ArgumentError, "Index is out of range.");

			_entries.RemoveAt(index);
		}

		public List<BookEntry> FindEntries(string name)
		{
			List<BookEntry> entries = new List<BookEntry>();
			foreach (BookEntry entry in _entries)
			{
				if (entry.Name.Contains(name))
					entries.Add(entry);
			}

			return entries;
		}
	}
}
