﻿using System;
using System.Collections.Generic;

namespace David.Academy.ProgrammingInCSharp.Exceptions
{
	class Program
	{
		static void Main(string[] args)
		{
			//BookDatabaseV1 bookDatabase1 = new BookDatabaseV1();
			//BookDatabaseV2 bookDatabase2 = new BookDatabaseV2();
			BookDatabaseV3 bookDatabase3 = new BookDatabaseV3();

			bool exit = false;
			do
			{
				Console.Write(@"
1. Show entries
2. Add an entry
3. Remove an entry
4. Find an entry
0. Exit

Enter your choice: ");

				ConsoleKeyInfo keyInfo = Console.ReadKey();
				Console.WriteLine();

				try
				{
					switch (keyInfo.KeyChar)
					{
						case '1':
							//ShowAllEntries(bookDatabase1);
							//ShowAllEntries(bookDatabase2);
							ShowAllEntries(bookDatabase3);
							break;
						case '2':
							//AddEntry(bookDatabase1);
							//AddEntry(bookDatabase2);
							AddEntry(bookDatabase3);
							break;
						case '3':
							//RemoveEntry(bookDatabase1);
							//RemoveEntry(bookDatabase2);
							RemoveEntry(bookDatabase3);
							break;
						case '4':
							//FindEntry(bookDatabase1);
							//FindEntry(bookDatabase2);
							FindEntry(bookDatabase3);
							break;
						case '0':
							exit = true;
							break;
					}
				}
				catch (Exception ex)
				{
					Console.WriteLine("An error has occurred: {0}", ex.Message);
				}
			}
			while (!exit);
		}

		#region BookDatabaseV1

		private static void ShowAllEntries(BookDatabaseV1 bookDatabase)
		{
			Console.WriteLine("Entries:");
			BookDatabaseStatus status;
			List<BookEntry> entries = bookDatabase.GetEntries(out status);
			if (status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred.", status);
				return;
			}
			ShowEntries(entries);
		}

		private static void AddEntry(BookDatabaseV1 bookDatabase)
		{
			Console.WriteLine("Add an entry:");

			Console.Write("ISBN (or empty to cancel): ");
			string isbn = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(isbn))
				return;

			Console.Write("Name: ");
			string name = Console.ReadLine();

			DateTime? issueDate;
			Console.Write("Issue date (e.g. {0}): ", DateTime.Now);
			string s = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(s))
				issueDate = null;
			else
				issueDate = DateTime.Parse(s);

			BookDatabaseStatus status;
			BookEntry entry = BookEntry.CreateEntry(isbn, name, out status);
			if (status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred.", status);
				return;
			}

			entry.IssueDate = issueDate;

			bookDatabase.AddEntry(entry, out status);
			if (status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred.", status);
				return;
			}
		}

		private static void RemoveEntry(BookDatabaseV1 bookDatabase)
		{
			Console.WriteLine("Remove an entry:");

			Console.Write("Index (or empty to cancel): ");
			string s = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(s))
				return;

			int index = int.Parse(s);

			BookDatabaseStatus status;
			bookDatabase.RemoveEntry(index, out status);
			if (status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred.", status);
				return;
			}
		}

		private static void FindEntry(BookDatabaseV1 bookDatabase)
		{
			Console.WriteLine("Find an entry:");

			Console.Write("Name (or empty to cancel): ");
			string name = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(name))
				return;

			Console.WriteLine("Found entries:");

			BookDatabaseStatus status;
			List<BookEntry> foundEntries = bookDatabase.FindEntries(name, out status);
			if (status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred.", status);
				return;
			}

			ShowEntries(foundEntries);
			Console.WriteLine("===");
		}

		#endregion

		#region BookDatabaseV2

		private static void ShowAllEntries(BookDatabaseV2 bookDatabase)
		{
			Console.WriteLine("Entries:");
			List<BookEntry> entries = bookDatabase.GetEntries();
			if (bookDatabase.Status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", bookDatabase.Status, bookDatabase.StatusText);
				return;
			}
			ShowEntries(entries);
		}

		private static void AddEntry(BookDatabaseV2 bookDatabase)
		{
			Console.WriteLine("Add an entry:");

			Console.Write("ISBN (or empty to cancel): ");
			string isbn = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(isbn))
				return;

			Console.Write("Name: ");
			string name = Console.ReadLine();

			DateTime? issueDate;
			Console.Write("Issue date (e.g. {0}): ", DateTime.Now);
			string s = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(s))
				issueDate = null;
			else
				issueDate = DateTime.Parse(s);

			BookDatabaseStatus status;
			BookEntry entry = BookEntry.CreateEntry(isbn, name, out status);
			if (status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred.", status);
				return;
			}

			entry.IssueDate = issueDate;

			bookDatabase.AddEntry(entry);
			if (bookDatabase.Status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", bookDatabase.Status, bookDatabase.StatusText);
				return;
			}
		}

		private static void RemoveEntry(BookDatabaseV2 bookDatabase)
		{
			Console.WriteLine("Remove an entry:");

			Console.Write("Index (or empty to cancel): ");
			string s = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(s))
				return;

			int index = int.Parse(s);

			bookDatabase.RemoveEntry(index);
			if (bookDatabase.Status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", bookDatabase.Status, bookDatabase.StatusText);
				return;
			}
		}

		private static void FindEntry(BookDatabaseV2 bookDatabase)
		{
			Console.WriteLine("Find an entry:");

			Console.Write("Name (or empty to cancel): ");
			string name = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(name))
				return;

			Console.WriteLine("Found entries:");

			List<BookEntry> foundEntries = bookDatabase.FindEntries(name);
			if (bookDatabase.Status != BookDatabaseStatus.Ok)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", bookDatabase.Status, bookDatabase.StatusText);
				return;
			}

			ShowEntries(foundEntries);
			Console.WriteLine("===");
		}

		#endregion

		#region BookDatabaseV3

		private static void ShowAllEntries(BookDatabaseV3 bookDatabase)
		{
			Console.WriteLine("Entries:");

			try
			{
				List<BookEntry> entries = bookDatabase.GetEntries();
				ShowEntries(entries);
			}
			catch (BookDatabaseException ex)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", ex.Status, ex.Message);
			}
		}

		private static void AddEntry(BookDatabaseV3 bookDatabase)
		{
			Console.WriteLine("Add an entry:");

			Console.Write("ISBN (or empty to cancel): ");
			string isbn = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(isbn))
				return;

			Console.Write("Name: ");
			string name = Console.ReadLine();

			DateTime? issueDate;
			Console.Write("Issue date (e.g. {0}): ", DateTime.Now);
			string s = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(s))
				issueDate = null;
			else
				issueDate = DateTime.Parse(s);

			try
			{
				BookEntry entry = new BookEntry(isbn, name)
				{
					IssueDate = issueDate,
				};

				bookDatabase.AddEntry(entry);
			}
			catch (BookDatabaseException ex)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", ex.Status, ex.Message);
			}
		}

		private static void RemoveEntry(BookDatabaseV3 bookDatabase)
		{
			Console.WriteLine("Remove an entry:");

			Console.Write("Index (or empty to cancel): ");
			string s = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(s))
				return;

			int index = int.Parse(s);

			try
			{
				bookDatabase.RemoveEntry(index);
			}
			catch (BookDatabaseException ex)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", ex.Status, ex.Message);
			}
		}

		private static void FindEntry(BookDatabaseV3 bookDatabase)
		{
			Console.WriteLine("Find an entry:");

			Console.Write("Name (or empty to cancel): ");
			string name = Console.ReadLine();
			if (string.IsNullOrWhiteSpace(name))
				return;

			try
			{
				Console.WriteLine("Found entries:");

				List<BookEntry> foundEntries = bookDatabase.FindEntries(name);

				ShowEntries(foundEntries);
				Console.WriteLine("===");
			}
			catch (BookDatabaseException ex)
			{
				Console.WriteLine("An error '{0}' has occurred: {1}", ex.Status, ex.Message);
			}
		}

		#endregion

		private static void ShowEntries(IEnumerable<BookEntry> entries)
		{
			int i = 0;
			foreach (BookEntry entry in entries)
			{
				Console.WriteLine("{0}. ISBN: {1} ; Name: {2} ; Issue date = {3}",
					i, entry.Isbn, entry.Name, entry.IssueDate);
			}
		}
	}
}
