﻿using System;
using System.Windows.Forms;

namespace David.Academy.ProgrammingInCSharp.Greeter
{
	/// <summary>
	/// Клас за главна форма на приложението
	/// </summary>
	public partial class GreeterForm : Form
	{
		// Частно поле за съхранение на форматиращия низ за текущо избрания поздрав
		private string _greeting;

		/// <summary>
		/// Конструктор на главната форма - автоматично генериран от Visual Studio
		/// </summary>
		public GreeterForm()
		{
			// Извикване на метода, съдържащ автоматично генерирания от дизайнера на Visual Studio код
			InitializeComponent();
		}

		/// <summary>
		/// Обновява състоянието на контролите след значимо действие на потребителя
		/// </summary>
		private void UpdateControls()
		{
			// Разрешава се употребата на бутона btn_Greet, едиствено когато е въведено непразно име
			// и е избран език за поздрава
			btn_Greet.Enabled = (!string.IsNullOrWhiteSpace(tb_Name.Text)
								 && !string.IsNullOrEmpty(_greeting));
		}

		/// <summary>
		/// Обработчик на събитието Click на бутона btn_Greet
		/// </summary>
		/// <param name="sender">Контролата, която е предизвикала събитието</param>
		/// <param name="e">Допълнителни аргументи на събитието (празни)</param>
		private void btn_Greet_Click(object sender, EventArgs e)
		{
			if (!string.IsNullOrEmpty(_greeting))
				MessageBox.Show(string.Format(_greeting, tb_Name.Text));
		}

		/// <summary>
		/// Обработчик на събитието CheckedChanged на радио бутона rb_Bulgarian
		/// </summary>
		/// <param name="sender">Контролата, която е предизвикала събитието</param>
		/// <param name="e">Допълнителни аргументи на събитието (празни)</param>
		private void rb_Bulgarian_CheckedChanged(object sender, EventArgs e)
		{
			if (((RadioButton)sender).Checked)
				_greeting = "Здравей, {0}!";
			UpdateControls();
		}

		/// <summary>
		/// Обработчик на събитието CheckedChanged на радио бутона rb_English
		/// </summary>
		/// <param name="sender">Контролата, която е предизвикала събитието</param>
		/// <param name="e">Допълнителни аргументи на събитието (празни)</param>
		private void rb_English_CheckedChanged(object sender, EventArgs e)
		{
			if (((RadioButton)sender).Checked)
				_greeting = "Greetings, {0}!";
			UpdateControls();
		}

		/// <summary>
		/// Обработчик на събитието CheckedChanged на радио бутона rb_Russian
		/// </summary>
		/// <param name="sender">Контролата, която е предизвикала събитието</param>
		/// <param name="e">Допълнителни аргументи на събитието (празни)</param>
		private void rb_Russian_CheckedChanged(object sender, EventArgs e)
		{
			if (((RadioButton)sender).Checked)
				_greeting = "Привет, {0}!";
			UpdateControls();
		}

		/// <summary>
		/// Обработчик на събитието CheckedChanged на радио бутона rb_German
		/// </summary>
		/// <param name="sender">Контролата, която е предизвикала събитието</param>
		/// <param name="e">Допълнителни аргументи на събитието (празни)</param>
		private void rb_German_CheckedChanged(object sender, EventArgs e)
		{
			if (((RadioButton)sender).Checked)
				_greeting = "Hallo, {0}!";
			UpdateControls();
		}

		/// <summary>
		/// Обработчик на събитието CheckedChanged на радио бутона rb_Japanese
		/// </summary>
		/// <param name="sender">Контролата, която е предизвикала събитието</param>
		/// <param name="e">Допълнителни аргументи на събитието (празни)</param>
		private void rb_Japanese_CheckedChanged(object sender, EventArgs e)
		{
			if (((RadioButton)sender).Checked)
				_greeting = "Konnichi wa, {0}!";
			UpdateControls();
		}

		/// <summary>
		/// Обработчик на събитието TextChanged на текстовото поле tb_Name
		/// </summary>
		/// <param name="sender">Контролата, която е предизвикала събитието</param>
		/// <param name="e">Допълнителни аргументи на събитието (празни)</param>
		private void tb_Name_TextChanged(object sender, EventArgs e)
		{
			UpdateControls();
		}
	}
}
