﻿namespace David.Academy.ProgrammingInCSharp.Lambdas
{
	/// <summary>
	/// Клас за колекция от книги
	/// </summary>
	class BookCollection
	{
		/// <summary>
		/// Делегат за обработчици на събитието CollectionFilled
		/// </summary>
		/// <param name="collection">Колекция от книги, която се е запълнила</param>
		public delegate void CollectionFilledHandler(BookCollection collection);

		// Частен масив от книги
		private readonly Book[] _books;
		// Частно поле за обработчиците на ръчно описано събитие
		//private CollectionFilledHandler _collectionFilled;

		// Ръчно описано събитие
		//public event CollectionFilledHandler CollectionFilled
		//{
		//    add { _collectionFilled += value; }
		//    remove { _collectionFilled -= value; }
		//}

		/// <summary>
		/// Текущ брой на книгите в колекцията
		/// </summary>
		public int Count { get; private set; }

		/// <summary>
		/// Събитие, което се предизвиква, когато колекцията се запълни
		/// </summary>
		public event CollectionFilledHandler CollectionFilled;

		/// <summary>
		/// Конструктор на колекцията от книги с параметър капацитет
		/// </summary>
		/// <param name="capacity">Капацитет на колекцията</param>
		public BookCollection(int capacity)
		{
			_books = new Book[capacity];
		}

		/// <summary>
		/// Добавя книга в колекцията
		/// </summary>
		/// <param name="book">Книга, която трябва да бъде добавена в колекцията</param>
		public void AddBook(Book book)
		{
			// Ако колекцията вече е запълнена, книгата не се добавя
			if (Count == _books.Length)
				return;

			// Добавяне на книгата в колекцията
			_books[Count++] = book;

			// Ако колекцията е запълнена след добавянето на книгата, се предизвиква събитието
			// за запълване на колекцията
			if (Count == _books.Length && CollectionFilled != null)
				CollectionFilled(this);
		}
	}
}
