﻿using System;

namespace David.Academy.ProgrammingInCSharp.GoldDiggers
{
	internal class Program
	{
		#region Nested type: FieldType

		private enum FieldType
		{
			None,
			OurGuy,
			SomeGuy,
			Diamond,
			Ground,
			Grass,
			Tree,
			Stone,
		}

		#endregion

		#region Nested type: Direction

		private enum Direction
		{
			None,
			Up,
			Down,
			Left,
			Right,
		}

		#endregion

		/// <summary>
		/// Взима размерите на правоъгълна матрица
		/// </summary>
		/// <param name="maxX">Максимална широчина</param>
		/// <param name="maxY">Максимална височина</param>
		/// <param name="x">Въведената широчина</param>
		/// <param name="y">Въведената височина</param>
		/// <returns>true - ако потребителят е въвел стойности, иначе - false</returns>
		private static bool GetFieldBounds(int maxX, int maxY, out int x, out int y)
		{
			x = 0;
			y = 0;

			#region Въвеждане на числото x

			while (true)
			{
				Console.Write("Въведете стойност за X ∈ [10; {0}] = ", maxX);
				string s = Console.ReadLine();
				if (string.IsNullOrEmpty(s))
					return false;

				if (!int.TryParse(s, out x))
				{
					Console.WriteLine("Моля, въведете цяло число.");
					continue;
				}

				if (x >= 10 && x <= maxX)
				{
					break;
				}

				Console.WriteLine("Моля, въведете цяло число в посочения интервал.");
			}

			#endregion

			#region Въвеждане на числото y

			while (true)
			{
				Console.Write("Въведете стойност за Y ∈ [10; {0}] = ", maxY);
				string s = Console.ReadLine();
				if (string.IsNullOrEmpty(s))
					return false;

				if (!int.TryParse(s, out y))
				{
					Console.WriteLine("Моля, въведете цяло число.");
					continue;
				}

				if (y >= 10 && y <= maxY)
				{
					break;
				}

				Console.WriteLine("Моля, въведете цяло число в посочения интервал.");
			}

			#endregion

			return true;
		}

		/// <summary>
		/// Създава игрално поле с определени размери
		/// </summary>
		/// <param name="m">Широчина на игралното поле</param>
		/// <param name="n">Височина на игралното поле</param>
		/// <param name="diamondCount">брой диаманти на игралното поле</param>
		/// <param name="ourGuyX">X-позиция на нашия човек</param>
		/// <param name="ourGuyY">Y-позиция на нашия човек</param>
		/// <returns>попълнено игрално поле</returns>
		private static FieldType[,] CreateField(int m, int n, out int diamondCount, out int ourGuyX, out int ourGuyY)
		{
			// създаваме игралното поле
			FieldType[,] field = new FieldType[m, n];

			Random random = new Random();

			#region Слагаме "нашия" човек

			ourGuyX = random.Next(0, m);
			ourGuyY = random.Next(0, n);
			field[ourGuyX, ourGuyY] = FieldType.OurGuy;

			#endregion

			#region Слагаме "някакви" човеци

			int someGuyCount = (int)Math.Round(m * n * 0.05);
			for (int i = 0; i < someGuyCount; i++)
			{
				int someGuyX, someGuyY;

				do
				{
					someGuyX = random.Next(0, m);
					someGuyY = random.Next(0, n);
				}
				while (field[someGuyX, someGuyY] != FieldType.None);

				field[someGuyX, someGuyY] = FieldType.SomeGuy;
			}

			#endregion

			#region Слагаме диаманти

			diamondCount = (int)Math.Round(m * n * 0.10);
			for (int i = 0; i < diamondCount; i++)
			{
				int diamondX, diamondY;

				do
				{
					diamondX = random.Next(0, m);
					diamondY = random.Next(0, n);
				}
				while (field[diamondX, diamondY] != FieldType.None);

				field[diamondX, diamondY] = FieldType.Diamond;
			}

			#endregion

			#region Добавяме останалите елементи в полето

			for (int y = 0; y < n; y++)
			{
				for (int x = 0; x < m; x++)
				{
					if (field[x, y] != FieldType.None)
						continue;

					FieldType fieldType;
					int p = random.Next(100);

					// с вероятност 40% добавяме земя
					if (p < 40)
						fieldType = FieldType.Ground;
					// с вероятност 30% добавяме трева
					else if (p < 70)
						fieldType = FieldType.Grass;
					// с вероятност 20% добавяме дърво
					else if (p < 90)
						fieldType = FieldType.Tree;
					// с вероятност (останалите) 10% добавяме камък
					else
						fieldType = FieldType.Stone;

					field[x, y] = fieldType;
				}
			}

			#endregion

			return field;
		}

		/// <summary>
		/// Изрисува на екрана подадено игрално поле
		/// </summary>
		/// <param name="field">игрално поле</param>
		private static void DrawField(FieldType[,] field)
		{
			int maxX = field.GetLength(0);
			int maxY = field.GetLength(1);

			Console.SetCursorPosition(0, 0);
			ConsoleColor color = Console.ForegroundColor;
			for (int y = 0; y < maxY; y++)
			{
				for (int x = 0; x < maxX; x++)
				{
					switch (field[x, y])
					{
						case FieldType.Ground:
							Console.ForegroundColor = ConsoleColor.DarkYellow;
							Console.Write('\u2592');
							break;
						case FieldType.Grass:
							Console.ForegroundColor = ConsoleColor.Green;
							Console.Write('\u2593');
							break;
						case FieldType.Tree:
							Console.ForegroundColor = ConsoleColor.DarkGreen;
							Console.Write('\u2663');
							break;
						case FieldType.Stone:
							Console.ForegroundColor = ConsoleColor.DarkGray;
							Console.Write('\u0665');
							break;
						case FieldType.Diamond:
							Console.ForegroundColor = ConsoleColor.Gray;
							Console.Write('\u2666');
							break;
						case FieldType.SomeGuy:
							Console.ForegroundColor = ConsoleColor.Red;
							Console.Write('\u263b');
							break;
						case FieldType.OurGuy:
							Console.ForegroundColor = ConsoleColor.White;
							Console.Write('\u263a');
							break;
						case FieldType.None:
							Console.Write(' ');
							break;
					}
				}

				Console.WriteLine();
			}

			Console.ForegroundColor = color;
		}

		/// <summary>
		/// Мести човек по игралното поле
		/// </summary>
		/// <param name="field">игрално поле</param>
		/// <param name="direction">посока на преместване</param>
		/// <param name="x">текуща X-координата</param>
		/// <param name="y">текуща Y-координата</param>
		/// <param name="diamondCount">брой диаманти на игралното поле</param>
		/// <returns>true - ако човекът се е преместил, иначе - false</returns>
		private static bool MoveGuy(FieldType[,] field, Direction direction,
			ref int x, ref int y, ref int diamondCount)
		{
			int newX, newY;

			#region Изчислява новите координати

			switch (direction)
			{
				case Direction.Up:
					newX = x;
					newY = y - 1;
					break;
				case Direction.Down:
					newX = x;
					newY = y + 1;
					break;
				case Direction.Left:
					newX = x - 1;
					newY = y;
					break;
				case Direction.Right:
					newX = x + 1;
					newY = y;
					break;
				default:
					return false;
			}

			#endregion

			// проверяваме дали новата позиция не е извън екрана
			if (newX < 0 || newX >= field.GetLength(0))
				return false;
			if (newY < 0 || newY >= field.GetLength(1))
				return false;

			FieldType fieldType = field[newX, newY];
			// според това какво има на новата позиция:
			switch (fieldType)
			{
				case FieldType.Ground:
				case FieldType.Grass:
				case FieldType.Diamond:
					field[newX, newY] = field[x, y];
					field[x, y] = FieldType.Ground;

					x = newX;
					y = newY;

					if (fieldType == FieldType.Diamond)
						diamondCount--;
					return true;
				default:
					return false;
			}
		}

		/// <summary>
		/// Мести някакви човеци в подадено игрално поле
		/// </summary>
		/// <param name="field">игрално поле</param>
		private static void MoveSomeGuys(FieldType[,] field, ref int diamondCount)
		{
			int maxX = field.GetLength(0);
			int maxY = field.GetLength(1);

			Random random = new Random();

			for (int y = 0; y < maxY; y++)
			{
				for (int x = 0; x < maxX; x++)
				{
					if (field[x, y] == FieldType.SomeGuy)
					{
						Direction direction = (Direction)random.Next(5);

						int newX = x, newY = y;
						MoveGuy(field, direction, ref newX, ref newY, ref diamondCount);
					}
				}
			}
		}

		/// <summary>
		/// Премества нашия човек по игралното поле в определена посока
		/// </summary>
		/// <param name="field">игрално поле</param>
		/// <param name="direction">посока на преместване</param>
		/// <param name="ourGuyX">X-позиция на нашия човек</param>
		/// <param name="ourGuyY">Y-позиция на нашия човек</param>
		private static void MoveOurGuy(FieldType[,] field, Direction direction, ref int diamondCount, ref int ourGuyX, ref int ourGuyY)
		{
			MoveGuy(field, direction, ref ourGuyX, ref ourGuyY, ref diamondCount);
		}

		/// <summary>
		/// Обработва входа от клавиатурата
		/// </summary>
		/// <param name="field">игрално поле</param>
		/// <param name="direction">посока на преместване</param>
		/// <param name="exits">потребителят иска да излезе от играта</param>
		/// <returns>true, ако потребителят е натиснал клавиш, иначе - false</returns>
		private static bool ProcessInput(ref Direction direction, out bool exits)
		{
			exits = false;
			if (!Console.KeyAvailable)
				return false;

			ConsoleKeyInfo info = Console.ReadKey(true);
			switch (info.Key)
			{
				case ConsoleKey.UpArrow:
					direction = Direction.Up;
					break;
				case ConsoleKey.DownArrow:
					direction = Direction.Down;
					break;
				case ConsoleKey.LeftArrow:
					direction = Direction.Left;
					break;
				case ConsoleKey.RightArrow:
					direction = Direction.Right;
					break;
				case ConsoleKey.Escape:
					exits = true;
					break;
			}

			return true;
		}

		static void Main(string[] args)
		{
			int maxX = Console.WindowWidth - 2;
			int maxY = Console.WindowHeight - 2;

			Console.Clear();

			int x, y;
			if (!GetFieldBounds(maxX, maxY, out x, out y))
				return;

			Console.Clear();

			int diamondCount, ourGuyX, ourGuyY;
			FieldType[,] field = CreateField(x, y, out diamondCount, out ourGuyX, out ourGuyY);

			bool exits = false;
			do
			{
				DrawField(field);
				MoveSomeGuys(field, ref diamondCount);

				Direction direction = Direction.None;
				bool isKeyPressed = ProcessInput(ref direction, out exits);
				if (isKeyPressed)
				{
					MoveOurGuy(field, direction, ref diamondCount, ref ourGuyX, ref ourGuyY);
				}

				System.Threading.Thread.Sleep(100);
			}
			while (!exits && diamondCount > 0);
		}
	}
}
